class DataPipeline:
    """
    Pipeline de traitement des données: collecte, transformation et export.
    """
    def __init__(self):
        pass

    def push_raw_data(self, data):
        """Envoie les données brutes dans le pipeline."""
        pass

    def transform(self, data):
        """Transforme les données collectées."""
        pass

    def export(self, data):
        """Exporte les données transformées vers l'API externe."""
        pass
